/*
# IOzHone performs simple minded testing of read() and write() speeds in
# perl, and was inspired by "iozone" but is in no way related to it.
#
# Basically IOzHone writes/reads a large file using a variety of
# blocksizes and performs timings on how fast each file is
# read/written. It is assumed that the file is of such size that the
# read/write operations are non-trivial and that the elapsed time for
# the operations can be measured in seconds.
#
# You may redistribute this code freely under the terms of
# version 2 of the GNU General Public License, available upon request
# from the Author or via FTP from "prep.ai.mit.edu".
#
# Please refer any patches or improvements to the Author.
#
# IOzHone.pl v2.1 - copyright (c) Alec Muffett August 24 1995

This C version by Michael Marte. Runs without perl and thus avoids
interpretation overhead. 

marte@informatik.uni-muenchen.de

21 Dec 1995	Compiles and runs on Linux and TOS (PureC)
*/


#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifdef linux
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#endif


#ifdef __PUREC__
#include <ext.h>
#include <tos.h>
#endif


/* 4 runs of reading/writing each file */
int runs = 4;

/* testfile to read/write */
char *testfile = "delete-me";

/* read/write a 16Mb for each blocksize */
long size = 8 * ((long) 1024 * 1024);

/* log device */
const char *logfile =
#ifdef linux
"/dev/tty"
#endif
#ifdef __PUREC__
"CON:"
#endif
 ;



long blocksizes[] =
{512, 1024, 2048, 4096, 8192, 16384, 32768, 65536};
char block[65536];

const char *title[] =
{"IOzHone v2.1 - Alec.Muffett@UK.Sun.COM, 24 Aug 1995",
 "This C version by marte@informatik.uni-muenchen.de"};


void
error (const char *place)
{
  fprintf (stderr, "%s: %s\n", place, strerror (errno));
  exit (1);
}


void
usage (const char *prgname)
{
  fprintf (stderr, "Usage: %s [OPTION]...\n\n"
	   "Options:\n"
	   "\t-h, --help\n"
	   "\t-o, --output path to log device\n"
	   "\t-f, --file path to file to hold test data\n"
	   "\t-s, --size number of bytes to write or read in each run\n"
	   "\t-r, --runs number of runs\n", prgname);
  exit (1);
}


int
main (int argc, char **argv)
{
  int i;
  const int tests = sizeof (blocksizes) / sizeof (long);
#define TESTS	64
  long write_bytes[TESTS], read_bytes[TESTS];
  clock_t write_elapsed[TESTS], read_elapsed[TESTS];
  FILE *log;

  for (i = 1; i < argc; i++)
    {
      if (strcmp (argv[i], "-h") == 0 || strcmp (argv[i], "--help") == 0)
	{
	  usage (argv[0]);
	}
      else if (strcmp (argv[i], "-r") == 0 || strcmp (argv[i], "--runs") == 0)
	{
	  if (i + 1 == argc)
	    usage (argv[0]);
	  i++;
	  runs = atoi (argv[i]);
	}
      else if (strcmp (argv[i], "-f") == 0 || strcmp (argv[i], "--file") == 0)
	{
	  if (i + 1 == argc)
	    usage (argv[0]);
	  i++;
	  testfile = argv[i];
	}
      else if (strcmp (argv[i], "-o") == 0 || strcmp (argv[i], "--output") == 0)
	{
	  if (i + 1 == argc)
	    usage (argv[0]);
	  i++;
	  logfile = argv[i];
	}
      else if (strcmp (argv[i], "-s") == 0 || strcmp (argv[i], "--size") == 0)
	{
	  if (i + 1 == argc)
	    usage (argv[0]);
	  i++;
	  size = atol (argv[i]);
	}
    }

  log = fopen (logfile, "w");
  if (!log)
    error ("open");

  fprintf (log, "%s\n%s\n\n", title[0], title[1]);

#ifdef linux
/*
Could someone be so kind to make the -o option work for this?
I don't know how to make the log device the stdout for system.
*/
  printf ("System Type:\n");
  system ("uname -a");
  printf ("\n\n");
  printf ("Current Directory:\n");
  system ("pwd");
  printf ("\n\n");
  printf ("Current Filesystem:\n");
  system ("df -ak .");
#endif

#ifdef __PUREC__
  fprintf (log, "System Type: TOS\n");
#endif

  fprintf (log, "\n\nfile to hold test data = %s\nsize = %ld\nruns = %d\n",
	   testfile, size, runs);

  fprintf (log, "...starting in 5 seconds...\n");
  sleep (5);

  for (i = 0; i < tests; i++)
    {
      long blocksize = blocksizes[i];
      long rwcount = size / blocksize;
      int j;

      write_bytes[i] = 0;
      write_elapsed[i] = 0;
      read_bytes[i] = 0;
      read_elapsed[i] = 0;

      fprintf (log, "\n%ldx%ldb:\t%d runs of writing: ", rwcount, blocksize, runs);
      for (j = 0; j < runs; j++)
	{
	  int fid = open (testfile, O_RDWR | O_CREAT
#ifdef linux
			  ,S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
#endif
	  );
	  long counter = rwcount;
	  clock_t start, finish, elapsed;

	  if (fid == -1)
	    error ("open");

	  start = clock ();
	  while (counter--)
	    if (write (fid, block, blocksize) != blocksize)
	      error ("write");
	  finish = clock ();
	  close (fid);

	  elapsed = finish - start;
	  write_bytes[i] += size;
	  write_elapsed[i] += elapsed;

	  fprintf (log, "%ld ", elapsed);

	}

      fprintf (log, "\n%ldx%ldb:\t%d runs of reading: ", rwcount, blocksize, runs);
      for (j = 0; j < runs; j++)
	{
	  int fid = open (testfile, O_RDWR | O_CREAT);
	  long counter = rwcount;
	  time_t start, finish, elapsed;

	  if (fid == -1)
	    error ("open");

	  start = clock ();
	  while (counter--)
	    if (read (fid, block, blocksize) != blocksize)
	      error ("read");
	  finish = clock ();
	  close (fid);

	  elapsed = finish - start;
	  read_bytes[i] += size;
	  read_elapsed[i] += elapsed;

	  fprintf (log, "%ld ", elapsed);
	}

      unlink (testfile);

    }

  fprintf (log, "\n\nResults:\n");

  for (i = 0; i < tests; i++)
    {
      long wspeed, rspeed;
      if (write_elapsed[i])
	wspeed = CLK_TCK * ((double) write_bytes[i]) / ((double) write_elapsed[i]);
      if (read_elapsed[i])
	rspeed = CLK_TCK * ((double) read_bytes[i]) / ((double) read_elapsed[i]);
      fprintf (log, "blocksize %ld  read=%ld bytes/s  write=%ld bytes/s\n",
	       blocksizes[i], rspeed, wspeed);
    }


  return 0;
}
