#!/usr/bin/perl
###
# Usage: iozhone [filename [filesize [runs]]]
#
# IOzHone performs simple minded testing of read() and write() speeds in
# perl, and was inspired by "iozone" but is in no way related to it.
#
# Basically IOzHone writes/reads a large file using a variety of
# blocksizes and performs timings on how fast each file is
# read/written. It is assumed that the file is of such size that the
# read/write operations are non-trivial and that the elapsed time for
# the operations can be measured in seconds.
#
# This code is You may redistribute this code freely under the terms of
# version 2 of the GNU General Public License, available upon request
# from the Author or via FTP from "prep.ai.mit.edu".
#
# Please refer any patches or improvements to the Author.
#
# IOzHone.pl v2.1 - copyright (c) Alec Muffett August 24 1995
###

# filename to read/write
$filename = shift || "delete-me";

# read/write a 16Mb for each blocksize
$filesize = shift || (16 * (1024**2));

# 6 runs of reading/writing each file
$runs = shift || 6;

@blocksizes = (512, 1024, 2048, 4096, 8192, 16384, 32768, 65536);

##################################################################
# should not be necessary to configure below this line
##################################################################

$title = 'IOzHone v2.1 - Alec.Muffett@UK.Sun.COM, 24 Aug 1995';

sub numerically { $a <=> $b }
@blocksizes = sort numerically @blocksizes; # just in case

$| = 1;                         # for stdout

$uname = `uname -a`;
$uname =~ s/\s+/ /go;

$cwd = `pwd`;
$cwd =~ s/\s+/ /go;

$df = `df -ak .`;

print "=" x length($title), "\n", $title, "\n", "=" x length($title), "\n\n";

print "System Type:\n$uname\n\n";
print "Current Directory:\n$cwd\n\n";
print "Current Filesystem:\n$df\n";

print "filename=$filename\n";
print "filesize=$filesize\n";
print "runs=$runs\n";
print "blocksizes=(@blocksizes)\n";
print "\n";

print "...starting in 5 seconds...\n\n";
sleep 5;                        # let the system settle down

foreach $blocksize (@blocksizes)
{
    $rwcount = int($filesize/$blocksize);
    $cookie = "${rwcount}x${blocksize}b";
    $block = "!" x $blocksize;  # make appropriately sized block

    printf("%-16s %d runs of writing: ", "$cookie:", $runs);
    foreach $run (1..$runs)
    {
	open(TEST, ">$filename") || die "$0: open: $filename: $!\n";
	$counter = $rwcount;

	$start = time;
	while ($counter--)
	{
	    unless(($retval = syswrite(TEST, $block, $blocksize, 0)) ==
		   $blocksize)
	    {
		die "$0: write: [$retval] $!\n";
	    }
	}
	$finish = time;
	close(TEST);

	$elapsed = $finish - $start;
	$write_bytes{$blocksize} += $filesize;
	$write_elapsed{$blocksize} += $elapsed;

	print "$elapsed ";
    }
    print "\n";

    printf("%-16s %d runs of reading: ", "$cookie:", $runs);
    foreach $run (1..$runs)
    {
	open(TEST, "$filename") || die "$0: open: $filename: $!\n";
	$counter = $rwcount;

	$start = time;
	while ($counter--)
	{
	    unless(($retval = sysread(TEST, $block, $blocksize, 0)) ==
		   $blocksize)
	    {
		die "$0: read: [$retval] $!\n";
	    }
	}
	$finish = time;
	close(TEST);

	$elapsed = $finish - $start;
	$read_bytes{$blocksize} += $filesize;
	$read_elapsed{$blocksize} += $elapsed;

	print "$elapsed ";
    }
    print "\n";
}

print "\n";
print "Results:\n";

foreach $blocksize (@blocksizes)
{
    $wspeed = "<divide by zero>";
    if ($write_elapsed{$blocksize})
    {
	$wspeed = int($write_bytes{$blocksize}/$write_elapsed{$blocksize});
    }

    $rspeed = "<divide by zero>";
    if ($read_elapsed{$blocksize})
    {
	$rspeed = int($read_bytes{$blocksize}/$read_elapsed{$blocksize});
    }

    printf("blocksize %-10s  read=%s bytes/s  write=%s bytes/s\n",
	   $blocksize, $rspeed, $wspeed);
}

print "=" x length($title), "\n";

unlink $filename || warn "$0: unlink: $filename: $!\n";

exit 0;
